#
# Name:    Makehead.sol
#
# Purpose: This file is used to customize operating system and installation
#          dependent features of the MX Tcl makefiles.  This version is
#          customized for Solaris 2.
#

#MXTCL_EXTRA_LIBRARIES =

MXTCL_EXTRA_LIBRARIES = -lezca -lca -lCom

#
#=========================================================================
#
# Generally, you should not have to modify anything after this point.
#

MAKEDEPEND = $(CC) -xM1 $(CFLAGS_EPICS) $(MX_APPINIT_INCLUDES) \
			mg_*.c mt_*.c mxAppInit.c > Makefile.dep

MX_TCL_SONAME       = libMxTcl.so.$(MX_MAJOR_VERSION)
MX_TCL_LIBRARY_NAME = $(MX_TCL_SONAME)

MX_TCL_STATIC_LIBRARY_NAME = libMxTcl.a

MX_TK_SONAME       = libMxTk.so.$(MX_MAJOR_VERSION)
MX_TK_LIBRARY_NAME = $(MX_TK_SONAME)

MX_TK_STATIC_LIBRARY_NAME = libMxTk.a

MX_APPINIT_FLAGS = $(CFLAGS)

MX_APPINIT_INCLUDES = 

MX_APPINIT_OUT   = -o

MX_TCL_LIB_OBJS  = $(MX_TCL_LIB_SRCS:.c=.o)

MX_TK_LIB_OBJS   = $(MX_TK_LIB_SRCS:.c=.o)

#-----------------------------------------------------------------

MX_TCL_LIBRARY_DELETE = rm $(MX_TCL_LIBRARY_NAME) libMxTcl.so

MX_TCL_LIBRARY_BUILD  = ld -G -o $(MX_TCL_LIBRARY_NAME) \
				$(MX_TCL_LIB_DIRS) $(MX_TCL_LIB_OBJS) \
				$(MX_TCL_LIBRARIES) -lc
MX_TCL_LIBRARY_BUILD2 = ln -sf $(MX_TCL_LIBRARY_NAME) libMxTcl.so

#-----------------------------------------------------------------

MX_TCL_STATIC_LIBRARY_DELETE = rm $(MX_TCL_STATIC_LIBRARY_NAME)

MX_TCL_STATIC_LIBRARY_BUILD  = ar rcs $(MX_TCL_STATIC_LIBRARY_NAME) \
						$(MX_TCL_LIB_OBJS)

#-----------------------------------------------------------------

MX_TK_LIBRARY_DELETE = rm $(MX_TK_LIBRARY_NAME) libMxTk.so

MX_TK_LIBRARY_BUILD  = ld -G -o $(MX_TK_LIBRARY_NAME) \
				$(MX_TK_LIB_DIRS) $(MX_TK_LIB_OBJS) \
				$(MX_TK_LIBRARIES) -lc
MX_TK_LIBRARY_BUILD2 = ln -sf $(MX_TK_LIBRARY_NAME) libMxTk.so

#-----------------------------------------------------------------

MX_TK_STATIC_LIBRARY_DELETE = rm $(MX_TK_STATIC_LIBRARY_NAME)

MX_TK_STATIC_LIBRARY_BUILD  = ar rcs $(MX_TK_STATIC_LIBRARY_NAME) \
						$(MX_TK_LIB_OBJS)

#-----------------------------------------------------------------

INCLUDES         = -I/usr/openwin/include -I$(LIBMXSRC) -I$(TCL_DIR)/include

MX_TCL_LIB_DIRS  = -L$(LIBMXSRC) -L$(TCL_DIR)/lib $(LIB_DIRS)
MX_TCL_LIBRARIES = -lMx $(LIBRARIES)

#-----------------------------------------------------------------

MXTCLSH_NAME = mxtclsh

MXTCLSH_OBJ_NAME = mxAppInit_tclsh.o

MXTCLSH_LIB_DIRS = $(LIB_DIRS) -L$(TCL_DIR)/lib

MXTCLSH_BUILD = $(CC) $(CFLAGS) -o $(MXTCLSH_NAME) $(MXTCLSH_OBJ_NAME) \
	$(MX_TCL_STATIC_LIBRARY_NAME) $(LIBMXSRC)/libMx.a \
	$(MXTCLSH_LIB_DIRS) -litcl3.0 -ltcl8.0 \
	$(MXTCL_EXTRA_LIBRARIES) -lsocket -lnsl -ldl -lposix4 -lm

#-----------------------------------------------------------------

MXWISH_NAME = mxwish

MXWISH_OBJ_NAME = mxAppInit_wish.o

MXWISH_LIB_DIRS = -L/usr/openwin/lib $(LIB_DIRS) -L$(TCL_DIR)/lib

MXWISH_BUILD = $(CC) $(CFLAGS) -o $(MXWISH_NAME) $(MXWISH_OBJ_NAME) \
	$(MX_TCL_STATIC_LIBRARY_NAME) $(MX_TK_STATIC_LIBRARY_NAME) \
	$(LIBMXSRC)/libMx.a $(MXWISH_LIB_DIRS) \
	-litk3.0 -litcl3.0 -ltk8.0 -ltcl8.0 \
	$(MXTCL_EXTRA_LIBRARIES) -lX11 -lsocket -lnsl -ldl -lposix4 -lm

#-----------------------------------------------------------------

MX_TCL_INSTALL  = -mkdir $(INSTALL_DIR)/bin
MX_TCL_INSTALL2 = -mkdir $(INSTALL_DIR)/etc
MX_TCL_INSTALL3 = -mkdir $(INSTALL_DIR)/etc/mxgui
MX_TCL_INSTALL4 = -mkdir $(INSTALL_DIR)/include
MX_TCL_INSTALL5 = -mkdir $(INSTALL_DIR)/lib
MX_TCL_INSTALL6 = -mkdir $(INSTALL_DIR)/lib/mxgui
MX_TCL_INSTALL7 = /usr/ucb/install -m 644 mx_tcl.h $(INSTALL_DIR)/include
MX_TCL_INSTALL8 = \
	/usr/ucb/install -m 644 $(MX_TCL_LIBRARY_NAME) $(INSTALL_DIR)/lib
MX_TCL_INSTALL9 = \
	/usr/ucb/install -m 644 $(MX_TCL_STATIC_LIBRARY_NAME) $(INSTALL_DIR)/lib
MX_TCL_INSTALL10 = -( cd $(INSTALL_DIR)/lib ; rm libMxTcl.so )
MX_TCL_INSTALL11 = \
	( cd $(INSTALL_DIR)/lib ; ln -sf $(MX_TCL_LIBRARY_NAME) libMxTcl.so )
MX_TCL_INSTALL12 = \
	/usr/ucb/install -m 644 $(MX_TK_LIBRARY_NAME) $(INSTALL_DIR)/lib
MX_TCL_INSTALL13 = \
	/usr/ucb/install -m 644 $(MX_TK_STATIC_LIBRARY_NAME) $(INSTALL_DIR)/lib
MX_TCL_INSTALL14 = -( cd $(INSTALL_DIR)/lib ; rm libMxTk.so )
MX_TCL_INSTALL15 = \
	( cd $(INSTALL_DIR)/lib ; ln -sf $(MX_TK_LIBRARY_NAME) libMxTk.so )
MX_TCL_INSTALL16 = /usr/ucb/install -m 644 mx.tcl $(INSTALL_DIR)/lib
MX_TCL_INSTALL17 = \
	/usr/ucb/install -m 644 ../mxgui/*.tcl $(INSTALL_DIR)/lib/mxgui
MX_TCL_INSTALL18 = /usr/ucb/install -m 755 mxtclsh $(INSTALL_DIR)/bin
MX_TCL_INSTALL19 = /usr/ucb/install -m 755 mxwish $(INSTALL_DIR)/bin
MX_TCL_INSTALL20 = /usr/ucb/install -m 755 ../mxgui/mxgui $(INSTALL_DIR)/bin
MX_TCL_INSTALL21 = chmod 755 $(INSTALL_DIR)/lib/mxgui/mxgui.tcl

