/*
 * Name:    mg_init.c
 *
 * Purpose: Initialization functions for the Tk interface to the MX library.
 *
 * Author:  William Lavender
 *
 *---------------------------------------------------------------------------
 *
 * Copyright 2000 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

#include <tcl.h>
#include <itcl.h>

#include "mx_util.h"
#include "mx_rec.h"
#include "mx_vers.h"
#include "mx_handl.h"
#include "mx_plot.h"
#include "mx_tcl.h"
#include "mx_tclpr.h"

#define USE_ITCL	TRUE

typedef struct {
	char name[80];
	Tcl_ObjCmdProc *fptr;
} MXTCL_COMMAND;

static MXTCL_COMMAND tcl_command[] = {
	{ "mxprivate_install_user_interrupt_handler",
					Mx_install_user_interrupt_handler_cmd },

	{ "mxprivate_install_output_handlers",
					Mx_install_output_handlers_cmd },
	{ "mxprivate_install_dialog_handler",
					Mx_install_dialog_handler_cmd },
};

static int num_tcl_commands = sizeof(tcl_command) / sizeof(tcl_command[0]);

static Tcl_Interp *Mxgui_main_interp;

/*-------------------------------------------------------*/

MX_EXPORT mx_status_type
mxtcl_process_tcl_events( unsigned long max_events )
{
	(void) Tcl_Eval( Mxgui_main_interp, "update" );

	return MX_SUCCESSFUL_RESULT;
}

/*-------------------------------------------------------*/

MX_EXPORT int
Mxtk_Init( Tcl_Interp *interp )
{
	char version[20];
	int i;

	Mxgui_main_interp = interp;

	/* Register ordinary Tcl commands. */

	for ( i = 0; i < num_tcl_commands; i++ ) {

		Tcl_CreateObjCommand( interp,
				tcl_command[i].name, tcl_command[i].fptr,
				(ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

	}

	/* Declare that we implement part of the Mx package. */

	sprintf(version, "%d.%d",
		mx_get_major_version(), mx_get_minor_version());

	Tcl_PkgProvide( interp, "Mx", version );

	/* Bring the global environment into the context of the libMxTcl
	 * library.  Using getenv() on any variable name seems to do this
	 * as a side effect.
	 */

	(void) getenv( "XYZZY" );

	return TCL_OK;
}

