#
#  Copyright 1999 Illinois Institute of Technology
#
#  See the file "LICENSE" for information on usage and redistribution
#  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

package provide MxGui 0.26

####################################################################
#                                                                  #
#     The following routine is the main routine for setup          #
#     and modification of xafs scans.                              #
#                                                                  #
####################################################################

proc mxgui_xafs_scan_setup { record_list scan_name \
				scan_record old_record_exists } {

# Create the toplevel widget for the scan setup screen.

	set window .xafs_scan_setup

	set status [ catch { toplevel $window } ]

# If the window already exists, bring it to the top.

	if { $status != 0 } {
		raise $window
		focus $window
		return
	}

# Hide the window for now.

	wm withdraw $window
	wm title $window "XAFS scan setup"

#
# Display the scan name and type at the top of the window.
#

	frame $window.name_frame
	pack $window.name_frame -padx 5 -pady 5 -expand yes -fill x

	label $window.name_frame.scan_name1 -text "Scan name:"
	pack $window.name_frame.scan_name1 -side left

	label $window.name_frame.scan_name2 -text $scan_name \
						-font $MxGui::monospaced_font
	pack $window.name_frame.scan_name2 -side left

	label $window.name_frame.pad -text " " -width 10
	pack $window.name_frame.pad -side left -expand yes -fill x

	label $window.name_frame.scan_type1 -text "Scan type:"
	pack $window.name_frame.scan_type1 -side left

	label $window.name_frame.scan_type2 -text "XAFS scan" \
						-font $MxGui::monospaced_font
	pack $window.name_frame.scan_type2 -side left

	frame $window.separator0 -height 2 -borderwidth 1 -relief sunken
	pack $window.separator0 -expand yes -fill both

#
# Initialize the display of the number of times to perform the scan.
#

	mxgui_initialize_num_scans_field $window $record_list \
				$scan_record $old_record_exists

	frame $window.separator1 -height 2 -borderwidth 1 -relief sunken
	pack $window.separator1 -expand yes -fill both

#
# Initialize the XAFS region display.
#

	mxgui_xafs_initialize_region_display \
			$window $record_list $scan_record \
			$old_record_exists

	frame $window.separator2 -height 2 -borderwidth 1 -relief sunken
	pack $window.separator2 -expand yes -fill both

#
# Setup the rest of the XAFS scan display fields.  The last argument to
# mxgui_initialize_common_scan_display is set to 0 since the integration
# times are specified as part of the regions.
#
	set show_measurement_parameters 0

	mxgui_initialize_standard_scan_display $window $record_list \
				$scan_record "xafs_scan" $old_record_exists \
				$show_measurement_parameters

#
# Display the OK and Cancel buttons.
#

	frame $window.b
	pack $window.b

	set buttonbox $window.b.buttonbox

	iwidgets::buttonbox $buttonbox -padx 0 -pady 2

	$buttonbox add OK -text "Create Scan" -command \
		[ code mxgui_xafs_create_scan $window $record_list \
			$scan_name $scan_record $old_record_exists ]

	$buttonbox add Cancel -text Cancel -command [ code destroy $window ]

	$buttonbox add Run -text "Create and Run Scan" -command \
		[ code mxgui_xafs_create_and_run_scan $window $record_list \
			$scan_name $scan_record $old_record_exists ]

	$buttonbox default OK

	pack $buttonbox -padx 10 -pady 10

#
# If this is an old scan, make the contents of the form match the
# parameters of the old scan.
#

	if { $old_record_exists != 0 } {
		mxgui_xafs_scan_copy_region_fields_from_record \
				$window $record_list $scan_record

		mxgui_scan_copy_input_device_fields_from_record \
				$window $record_list $scan_record

		mxgui_scan_copy_measurement_fields_from_record \
				$window $record_list $scan_record \
				$show_measurement_parameters

		mxgui_scan_copy_datafile_and_plot_fields_from_record \
				$window $record_list $scan_record
	}

#
# Make the window appear on the screen.
#
	wm deiconify $window

	update
}

#########################################################################

proc mxgui_xafs_initialize_region_display { window record_list scan_record \
						old_record_exists } {
#
# Display the list of regions.
#
	set num_regions_frame $window.num_regions_frame
	set region_frame      $window.region_frame

	frame $num_regions_frame
	pack $num_regions_frame -padx 5 -pady 5

	if { $old_record_exists == 0 } {
		set num_energy_regions   2
		set num_k_regions        2
	} else {
		set num_energy_regions \
			[ $scan_record get_field num_energy_regions ]

		set num_k_regions [ $scan_record get_field num_k_regions ]
	}

#
# First, we show the number of each kind of region.
#

	iwidgets::entryfield $num_regions_frame.num_energy_regions \
					-labeltext "Number of energy regions:"

	$num_regions_frame.num_energy_regions clear
	$num_regions_frame.num_energy_regions insert 0 $num_energy_regions

	$num_regions_frame.num_energy_regions configure -validate \
					{ iwidgets::Entryfield::integer %P }

	pack $num_regions_frame.num_energy_regions -side left

	iwidgets::entryfield $num_regions_frame.num_k_regions \
					-labeltext "Number of k regions:"

	$num_regions_frame.num_k_regions clear
	$num_regions_frame.num_k_regions insert 0 $num_k_regions

	$num_regions_frame.num_k_regions configure -validate \
					{ iwidgets::Entryfield::integer %P }

	pack $num_regions_frame.num_k_regions -side left

#
# old_num_energy_regions and old_num_k_regions are hidden fields used to 
# store previous values of these variables for use by other procedures.
# These labels are not intended to be visible, so don't pack or grid them.
#
	label $num_regions_frame.old_num_energy_regions -text 0
	label $num_regions_frame.old_num_k_regions -text 0
#
# Create the frame for the list of regions.
#
	frame $region_frame
	pack $region_frame -padx 5 -pady 5
#
# Put labels at the top of the regions list.
#
	label $region_frame.boundary -text "Region Boundary"
	label $region_frame.units1 -width 5
	label $region_frame.stepsize -text "Step Size"
	label $region_frame.units2 -width 5
	label $region_frame.integration_time -text "Integration Time"

	grid $region_frame.boundary $region_frame.units1 \
		$region_frame.stepsize $region_frame.units2 \
		$region_frame.integration_time
#
# Set up the individual region information.
#
	$num_regions_frame.num_energy_regions configure -command [ code \
		mxgui_xafs_scan_redo_region_fields \
			$region_frame $num_regions_frame \
			$record_list $scan_record $old_record_exists ]

	$num_regions_frame.num_k_regions configure -command [ code \
		mxgui_xafs_scan_redo_region_fields \
			$region_frame $num_regions_frame \
			$record_list $scan_record $old_record_exists ]
#
# Initialize the region display.
#
	mxgui_xafs_scan_redo_region_fields $region_frame $num_regions_frame \
			$record_list $scan_record $old_record_exists
}

########################################################################

proc mxgui_xafs_scan_redo_region_fields { region_frame num_regions_frame \
			record_list scan_record old_record_exists } {

	set old_widget [ itcl::find objects $region_frame.start ]

	if { $old_widget == "" } {
		iwidgets::entryfield $region_frame.start

		label $region_frame.start_units1

		grid $region_frame.start -row 1 -column 0

		grid $region_frame.start_units1 -row 1 -column 1 -sticky w
	}
#
# Find out how the number of regions has changed.
#
	set old_num_energy_regions \
		[ $num_regions_frame.old_num_energy_regions cget -text ]

	set old_num_k_regions \
		[ $num_regions_frame.old_num_k_regions cget -text ]

	set num_energy_regions [ $num_regions_frame.num_energy_regions get ]

	set num_k_regions [ $num_regions_frame.num_k_regions get ]

	if { $num_energy_regions > 0 } {
		$region_frame.start_units1 configure -text "eV"
	} else {
		$region_frame.start_units1 configure -text "A-1"
	}

#
# Redo the list of energy regions.
#

	if { $num_energy_regions > $old_num_energy_regions } {

		# Move down the k region fields to make room for the new
		# energy regions.

		for { set i [ expr $old_num_k_regions - 1 ] } \
				{ $i >= 0 } { incr i -1 } {

			set thisregion $region_frame.k$i

			set newrow [ expr 2 * ($num_energy_regions \
				+ $old_num_k_regions + $i - 1) + 1 ]

			grid ${thisregion}_end -row $newrow -column 0
			grid ${thisregion}_units1 -row $newrow -column 1 \
								-sticky w

			set newrow [ expr $newrow - 1 ]

			grid ${thisregion}_stepsize -row $newrow -column 2
			grid ${thisregion}_units2 -row $newrow -column 3 \
								-sticky w
			grid ${thisregion}_integration_time \
						-row $newrow -column 4
		}

		# Add new energy regions to the list.

		for { set i $old_num_energy_regions } \
				{ $i < $num_energy_regions } { incr i } {

			set thisregion $region_frame.energy$i

			set thisrow [ expr 2 * ($i + 1) ]

			iwidgets::entryfield ${thisregion}_stepsize

			label ${thisregion}_units2

			iwidgets::entryfield ${thisregion}_integration_time

			grid ${thisregion}_stepsize -row $thisrow -column 2
			grid ${thisregion}_units2 -row $thisrow -column 3 \
								-sticky w
			grid ${thisregion}_integration_time \
						-row $thisrow -column 4

			set thisrow [ expr $thisrow + 1 ]

			iwidgets::entryfield ${thisregion}_end

			label ${thisregion}_units1

			grid ${thisregion}_end -row $thisrow -column 0
			grid ${thisregion}_units1 -row $thisrow -column 1 \
								-sticky w

			${thisregion}_units1 configure -text "eV"
			${thisregion}_units2 configure -text "eV"
		}
	} else {

		# Delete old energy regions from the list.

		for { set i $num_energy_regions } \
				{ $i < $old_num_energy_regions } { incr i } {

			set thisregion $region_frame.energy$i

			destroy ${thisregion}_end
			destroy ${thisregion}_units1
			destroy ${thisregion}_stepsize
			destroy ${thisregion}_units2
			destroy ${thisregion}_integration_time
		}

		# Move up the k region fields to fill in the gap left by
		# deleting the energy regions.

		for { set i 0 } { $i < $old_num_k_regions } { incr i } {

			set thisregion $region_frame.k$i

			set newrow [ expr 2 * ($num_energy_regions + $i + 1) ]

			grid ${thisregion}_stepsize -row $newrow -column 2
			grid ${thisregion}_units2 -row $newrow -column 3 \
								-sticky w
			grid ${thisregion}_integration_time \
						-row $newrow -column 4

			set newrow [ expr $newrow + 1 ]

			grid ${thisregion}_end -row $newrow -column 0
			grid ${thisregion}_units1 -row $newrow -column 1 \
								-sticky w
		}
	}

#
# Redo the list of k regions.
#

	if { $num_k_regions > $old_num_k_regions } {

		# Add new k regions to the list.

		for { set i $old_num_k_regions } \
				{ $i < $num_k_regions } { incr i } {

			set thisregion $region_frame.k$i

			set thisrow [ expr 2 * ($num_energy_regions + $i + 1) ]

			iwidgets::entryfield ${thisregion}_stepsize

			label ${thisregion}_units2

			iwidgets::entryfield ${thisregion}_integration_time

			grid ${thisregion}_stepsize -row $thisrow -column 2
			grid ${thisregion}_units2 -row $thisrow -column 3 \
								-sticky w
			grid ${thisregion}_integration_time \
						-row $thisrow -column 4

			set thisrow [ expr $thisrow + 1 ]

			iwidgets::entryfield ${thisregion}_end

			label ${thisregion}_units1

			grid ${thisregion}_end -row $thisrow -column 0
			grid ${thisregion}_units1 -row $thisrow -column 1 \
								-sticky w

			${thisregion}_units1 configure -text "A-1"
			${thisregion}_units2 configure -text "A-1"
		}
	} else {

		# Delete old k regions from the list.

		for { set i $num_k_regions } \
				{ $i < $old_num_k_regions } { incr i } {

			set thisregion $region_frame.k$i

			destroy ${thisregion}_end
			destroy ${thisregion}_units1
			destroy ${thisregion}_stepsize
			destroy ${thisregion}_units2
			destroy ${thisregion}_integration_time
		}
	}

#
# Save the new number of energy regions and k regions.
#

	$num_regions_frame.old_num_energy_regions configure \
					-text $num_energy_regions

	$num_regions_frame.old_num_k_regions configure -text $num_k_regions
}

proc mxgui_xafs_scan_copy_region_fields_from_record { window record_list \
						scan_record } {

	set num_energy_regions [ $scan_record get_field num_energy_regions ]
	set num_k_regions [ $scan_record get_field num_k_regions ]
	set boundary_list [ split [ $scan_record get_field region_boundary ] ]
	set stepsize_list [ split [ $scan_record get_field region_step_size ] ]
	set time_list \
		[ split [ $scan_record get_field region_measurement_time ] ]

	set region_frame $window.region_frame

	$region_frame.start insert end [ lindex $boundary_list 0 ]

	for { set i 0 } { $i < $num_energy_regions } { incr i } {

		set thisregion $region_frame.energy$i

		set end      [ lindex $boundary_list [ expr $i + 1 ] ]
		set stepsize [ lindex $stepsize_list $i ]
		set time     [ lindex $time_list $i ]

		${thisregion}_end              insert end $end
		${thisregion}_stepsize         insert end $stepsize
		${thisregion}_integration_time insert end $time
	}
	for { set i 0 } { $i < $num_k_regions } { incr i } {

		set thisregion $region_frame.k$i

		set j [ expr $num_energy_regions + $i ]

		set end      [ lindex $boundary_list [ expr $j + 1 ] ]
		set stepsize [ lindex $stepsize_list $j ]
		set time     [ lindex $time_list $j ]

		${thisregion}_end              insert end $end
		${thisregion}_stepsize         insert end $stepsize
		${thisregion}_integration_time insert end $time
	}
}

####################################################################
#                                                                  #
# The following routine sets up the description for the new scan.  #
#                                                                  #
####################################################################

proc mxgui_xafs_create_scan { window record_list scan_name scan_record \
				old_record_exists } {

	set description "$scan_name  scan xafs_scan_class xafs_scan \"\" \"\" "

# Append the number of times to run this scan.

	set num_scans [ $window.num_scans_frame.num_scans get ]

	append description "$num_scans 0 0 "

# Append the input devices, if any.

	set num_inputs_frame    $window.readout_frame.inputs.num_inputs_frame
	set input_device_frame  $window.readout_frame.inputs.input_device_frame

	set num_input_devices [ $num_inputs_frame.num_input_devices get ]

	append description "$num_input_devices "

	for { set i 0 } { $i < $num_input_devices } { incr i } {

		set input_device_name \
			[ $input_device_frame.device${i}_name get ]

		set name_length \
			[ string length [ string trim $input_device_name ] ]

		if { $name_length == 0 } {
			tk_dialog .motor_select_empty_input_device_name \
		"Empty input device name" \
		"No input device name was specified for input device $i." \
				error 0 "Done"

			return
		}
		append description "$input_device_name "
	}

# Append the after scan action (currently always 0).

	append description "0 "

# Append the timer information.

	set measurement_frame $window.readout_frame.measurement

	append description "[ $measurement_frame.sframe.settling_time get ] "

	append description "preset_time \"0 timer1\" "

# Append the datafile information.

	set datafile_frame $window.datafile_plot_frame.datafile_frame

	append description "[ string tolower [ $datafile_frame.type get ] ] "

	append description "[ $datafile_frame.name get ] "

# Append the plot information.

	set plot_frame $window.datafile_plot_frame.plotfile_frame

	append description "[ string tolower [ $plot_frame.type get ] ] "

	append description "\"[ $plot_frame.arguments get ]\" "

# Append the number of regions information.

	set num_energy_regions \
			[ $window.num_regions_frame.num_energy_regions get ]

	set num_k_regions [ $window.num_regions_frame.num_k_regions get ]

	set num_regions [ expr $num_energy_regions + $num_k_regions ]

	set num_boundaries [ expr $num_regions + 1 ]

	append description \
	  "$num_regions $num_energy_regions $num_k_regions $num_boundaries "

# Append the region information.

	append description "[ $window.region_frame.start get ] "

	list end_list
	list stepsize_list
	list time_list

	for { set i 0 } { $i < $num_energy_regions } { incr i } {

		lappend end_list [ $window.region_frame.energy${i}_end get ]

		lappend stepsize_list \
		    [ $window.region_frame.energy${i}_stepsize get ]

		lappend time_list \
		    [ $window.region_frame.energy${i}_integration_time get ]
	}

	for { set i 0 } { $i < $num_k_regions } { incr i } {

		lappend end_list [ $window.region_frame.k${i}_end get ]

		lappend stepsize_list \
		    [ $window.region_frame.k${i}_stepsize get ]

		lappend time_list \
		    [ $window.region_frame.k${i}_integration_time get ]
	}

	append description "$end_list $stepsize_list $time_list"

# Create the new scan record.

	if { $old_record_exists != 0 } {
		$scan_record delete_record
	}

	if [ catch { $record_list create_record_from_description \
					$description } new_record ] {

		set t "Attempt to create scan '$scan_name' failed."

		tk_dialog .mxgui_create_record_failed_dialog \
			"Record Creation Failed" $t error 0 "OK"

		return
	}

	if [ catch { $new_record finish_record_initialization } ] {

		set t "Attempt to initialize scan record '$scan_name' failed."

		tk_dialog .mxgui_finish_initialization_dialog \
			"Record Initialization Failed" $t error 0 "OK"

		return
	}

# Get rid of the scan setup window

	wm withdraw $window
	destroy $window

# Remember that we now have unsaved scans.

	$record_list configure -unsaved_scans 1
}

####################################################################
#                                                                  #
# The following routine creates a scan with mxgui_xafs_create_scan #
# and then runs the scan.                                          #
#                                                                  #
####################################################################

proc mxgui_xafs_create_and_run_scan { window record_list \
				scan_name scan_record old_record_exists } {

	mxgui_xafs_create_scan $window $record_list \
				$scan_name $scan_record $old_record_exists

	#
	#  Check to see if the files created by this scan will
	#  overwrite already existing files.
	#

	set status [ mxgui_check_for_datafile_name_collision $scan_record ]

	#
	#  If the returned status is 0, the user has decided to
	#  abort the scan since preexisting files would be
	#  overwritten.
	#

	if { $status == 0 } {
		return
	}

	#
	#  Prompt the user for scan header changes
	#

	set datafile_type [ $scan_record get_field datafile_type ]

	set status [ menubar_setup_scan_header_change_dialog \
				$record_list $datafile_type $scan_record ]

	#
	#  If the returned status is 1, we have to do the scan ourself.
	#  Otherwise, a callback that was set up in the procedure
	#  menubar_setup_scan_header_change_dialog will do it for us.
	#

	if { $status == 1 } {
		mxgui_perform_scan $scan_record
	}
}

